@@ -40,14 +40,17 @@ double gScaleRatio = 1.0;
 int gDisplayMode = PHO_DISPLAY_NORMAL;
 
 /* Slideshow delay is zero by default -- no slideshow. */
-int gDelaySeconds = 0;
+int gDelayMillis = 0;
 int gPendingTimeout = 0;
 
+/* Loop back to the first image after showing the last one */
+int gRepeat = 0;
+
 static int RotateImage(PhoImage* img, int degrees);    /* forward */
 
 static gint DelayTimer(gpointer data)
 {
-    if (gDelaySeconds == 0)    /* slideshow mode was cancelled */
+    if (gDelayMillis == 0)    /* slideshow mode was cancelled */
         return FALSE;
 
     if (gDebug) printf("-- Timer fired\n");
@@ -62,15 +65,15 @@ int ShowImage()
     ScaleAndRotate(gCurImage, 0);
     /* Keywords dialog will be updated if necessary from DrawImage */
 
-    if (gDelaySeconds > 0 && gPendingTimeout == 0
+    if (gDelayMillis > 0 && gPendingTimeout == 0
         && (gCurImage->next != 0 || gCurImage->next != gFirstImage)) {
-        if (gDebug) printf("Adding timeout for %d msec\n", gDelaySeconds*1000);
-        gPendingTimeout = g_timeout_add (gDelaySeconds * 1000, DelayTimer, 0);
+        if (gDebug) printf("Adding timeout for %d msec\n", gDelayMillis);
+        gPendingTimeout = g_timeout_add (gDelayMillis, DelayTimer, 0);
     }
 
     return 0;
 }
-	
+
 static int LoadImageFromFile(PhoImage* img)
 {
     GError* err = NULL;
@@ -160,63 +163,52 @@ int ThisImage()
     return 0;
 }
 
+/* Go to the next image after gCurImage,
+ * or the first image if gCurImage isn't set. If an image fails to load,
+ * delete it from the image list and move on to the next image.
+ */
 int NextImage()
 {
-    int retval = 0;
-    int looping = 0;
     if (gDebug)
         printf("\n================= NextImage ====================\n");
 
+    PhoImage* origCurImage = gCurImage;
+
     /* Loop, since images may fail to load
      * and may need to be deleted from the list
      */
     while (1)
     {
-        if (gFirstImage == 0)
+        if (! gFirstImage) {
             /* There's no list! How can we go to the next item? */
+            if (gDebug)
+                printf("NextImage: empty list!\n");
             return -1;
+        }
 
-        if (gCurImage == 0) {  /* no image loaded yet, first call */
+        if (! gCurImage) {
             if (gDebug)
                 printf("NextImage: going to first image\n");
             gCurImage = gFirstImage;
         }
-
-        else if (looping && gCurImage->next == gFirstImage)
-            /* We're to the end of the list, after deleting something bogus */
+        else if (gCurImage->next == gFirstImage) {  /* end of the list */
             return -1;
-
-        else if ((gCurImage->next == 0) || (gCurImage->next == gFirstImage))
-            /* We're at the end of the list, can't go farther.
-             * However, we may have gotten here by trying to go to
-             * the next image and failing, in which case we no longer
-             * have a pixmap loaded. So we still need to LoadImage,
-             * but we'll want to return -1 to indicate we didn't progress.
-             */
-            retval = -1;
-
-        /* We only want to go to ->next the first time;
-         * if we're looping because of an error, gCurImage is already set.
-         */
-        else if (!looping)
+        }
+        else {
             gCurImage = gCurImage->next;
+        }
 
         if (LoadImageAndRotate(gCurImage) == 0) {   /* Success! */
             ShowImage();
-            return retval;
+            return 0;
         }
 
-        /* The image didn't load. Remove it from the list.
-         * That means we're going to loop around and try again,
-         * so keep gCurImage where it is (but change gCurImage->next).
-         */
+        /* gCurImage failed to load. */
         if (gDebug)
             printf("Skipping '%s' (didn't load)\n", gCurImage->filename);
         DeleteItem(gCurImage);
-        looping = 1;
+        gCurImage = origCurImage;
     }
-    /* NOTREACHED */
-    return 0;
 }
 
 int PrevImage()
@@ -425,7 +417,7 @@ int ScaleAndRotate(PhoImage* img, int degrees)
             new_height = img->curHeight;
         }
     }
-        
+
     /* Fullscreen: Scale either up or down if necessary to make
      * the largest dimension match the screen size.
      */
@@ -601,7 +593,7 @@ void ReallyDelete(PhoImage* delImg)
 
     DeleteItem(delImg);
 
-    /* If we just deleted the last image, all we can do is quit */
+    /* If we just deleted the only image, all we can do is quit */
     if (!gFirstImage)
         EndSession();
 
@@ -729,17 +721,18 @@ void Usage()
 {
     printf("pho version %s.  Copyright 2002-2009 Akkana Peck akkana@shallowsky.com.\n", VERSION);
     printf("Usage: pho [-dhnp] image [image ...]\n");
-    printf("\t-p: Presentation mode (full screen, centered)\n");
+    printf("\t-p:  Presentation mode (full screen, centered)\n");
     printf("\t-p[resolution]: Projector mode:\n\tlike presentation mode but in upper left corner\n");
-    printf("\t-P: No presentation mode (separate window) -- default\n");
-    printf("\t-k: Keywords mode (show a Keywords dialog for each image)\n");
-    printf("\t-n: Replace each image window with a new window (helpful for some window managers)\n");
+    printf("\t-P:  No presentation mode (separate window) -- default\n");
+    printf("\t-k:  Keywords mode (show a Keywords dialog for each image)\n");
+    printf("\t-n:  Replace each image window with a new window (helpful for some window managers)\n");
     printf("\t-sN: Slideshow mode, where N is the timeout in seconds\n");
+    printf("\t-r:  Repeat: loop back to the first image after showing the last\n");
     printf("\t-cpattern: Caption/Comment file pattern, format string for reworking filename\n");
-    printf("\t--: Assume no more flags will follow\n");
-    printf("\t-d: Debug messages\n");
-    printf("\t-h: Help: Print this summary\n");
-    printf("\t-v: Verbose help: Print a summary of key bindings\n");
+    printf("\t--:  Assume no more flags will follow\n");
+    printf("\t-d:  Debug messages\n");
+    printf("\t-h:  Help: Print this summary\n");
+    printf("\t-v:  Verbose help: Print a summary of key bindings\n");
     exit(1);
 }
 
@@ -748,42 +741,35 @@ void VerboseHelp()
     printf("pho version %s.  Copyright 2002,2003,2004,2007 Akkana Peck akkana@shallowsky.com.\n", VERSION);
     printf("Type pho -h for commandline arguments.\n");
     printf("\npho Key Bindings:\n\n");
-    printf("<space>\tNext image (or cancel slideshow mode)\n");
-    printf("-\tPrevious image\n");
-    printf("<backspace>\tPrevious image\n");
+    printf("<space>, <Page Down>\n\tNext image (or cancel slideshow mode)\n");
+    printf("<backspace>, <Page Up>\n\tPrevious image\n");
     printf("<home>\tFirst image\n");
-    printf("f\tToggle fullscreen mode (scale even small images up to fullscreen)\n");
-    printf("F\tToggle full-size mode (even if bigger than screen)\n");
+    printf("f\tToggle full-size mode (even if bigger than screen)\n");
+    printf("F\tToggle fullscreen mode (scale even small images up to fullscreen)\n");
     printf("k\tTurn on keywords mode: show the keywords dialog\n");
     printf("p\tToggle presentation mode (take up the whole screen, centering the image)\n");
     printf("d\tDelete current image (from disk, after confirming with another d)\n");
     printf("0-9\tRemember image in note list 0 through 9 (to be printed at exit)\n");
-    printf("  (In keywords dialog, alt + 0-9 adds 10, e.g. alt-4 triggers flag 14.\n");
-    printf("t\tRotate right 90 degrees\n");
-    printf("r\tRotate right 90 degrees\n");
-    printf("<Right>\tRotate right 90 degrees\n");
-    printf("T\tRotate left 90 degrees\n");
-    printf("R\tRotate left 90 degrees\n");
-    printf("l\tRotate left 90 degrees\n");
-    printf("L\tRotate left 90 degrees\n");
-    printf("<left>\tRotate left 90 degrees\n");
+    printf("\t(In keywords dialog, alt + 0-9 adds 10, e.g. alt-4 triggers flag 14.\n");
+    printf("t, r, <Right>\n\tRotate right 90 degrees\n");
+    printf("T, R, l, L, <Left>]n\tRotate left 90 degrees\n");
     printf("<up>\tRotate 180 degrees\n");
-    printf("+\tDouble size\n");
-    printf("=\tDouble size\n");
-    printf("/\tHalf size\n");
-    printf("<kp>-\tHalf size\n");
+    printf("+, =\tDouble size\n");
+    printf("/, -\tHalf size\n");
     printf("i\tShow/hide info dialog\n");
-    printf("o\nChange the working file set (add files or make a new list)\n");
+    printf("o\tChange the working file set (add files or make a new list)\n");
     printf("g\tRun gimp on the current image\n");
-    printf("\t(or set PHO_REMOTE to an alternate command)\n");
+    printf("\t(or set PHO_CMD to an alternate command)\n");
     printf("q\tQuit\n");
     printf("<esc>\tQuit (or hide a dialog, if one is showing)\n");
-    exit(1);
+    printf("\n");
+    printf("Pho mouse bindings:\n");
+    printf("In presentation mode: drag with middlemouse to pan/move.\n");
+    printf("\n");
+    printf("Pho environment variables:\n");
+    printf("PHO_ARGS: default arguments (e.g. -p)\n");
+    printf("PHO_CMD : command to run when pressing g (default: gimp).\n");
+    printf("          Use an empty string if you don't want any command.\n");
+    exit(0);
 }
 
-
-
-
-
-
-
